/* qag.h, qag.cc                                                 */
/* version 0.9                                                   */
/* Stefan Bauberger (bauberger@physik.uni-wuerzburg.de           */
/* last revision 4.12.95                                         */
/* ------------------------------------------------------------- */
/* This is an adaption of the integration-routine QAG from the   */
/* QUADPACK-package for C++ and for use with complex functions   */
/* or other numerical types.                                     */
/* See:                                                          */
/*   R. Piessens, E. deDoncker-Kapenga, C. Uberhuber, D. Kahaner */
/*   doubledoublepack: a Subroutine Package for Automatic Integration    */
/*   Springer Verlag, 1983.                                      */
/* ------------------------------------------------------------- */
/* Purpose: adaptive gauss-kronrod-integration                   */
/*          of real or complex functions of a real variable      */
/* ------------------------------------------------------------- */ 
/* The code has been translated with f2c and then slightly       */
/* streamlined. It runs indepently of any f2c-header-file or     */
/* library.                                                      */
/* Note:                                                         */
/*   Some effort could still be put into making the code more    */
/*   c++-like. Any user is invited to do this.                   */
/* ------------------------------------------------------------- */
/* The calling program has to provide a function:                */
/*   RESULT_TYPE foo(Double x);                                    */
/* First initialize a integration class with:                    */
/* - QAGCLASS v;                                                 */
/*         in this case the integration will be evaluated with   */
/*         standard subdivisions setting.                        */
/* - QAGCLASS v(int error_type,Double error);                      */
/*         with error_type=FIX_RELATIVE_ERROR or                 */
/*         with error_type=FIX_ABSOLUTE_ERROR                    */
/* - QAGCLASS v(Double abs_error,Double rel_error,int max_subdiv);   */
/*         the routine will then stop when one of the conditions */
/*         is fulfilled                                          */
/* Calling sequence with a and b as integration limits:          */
/* RESULT_TYPE v.integrate(RESULT_TYPE (*foo) (Double),            */
/*                         Double a,Double b);                       */
/*           a and b are the integration limits                  */
/* ------------------------------------------------------------- */

#ifndef __QagClass__
#define __QagClass__

#define QMAXINT 600  // maximal number of subdivisions
                     // for double precision this is usually too large

// definitions for the variable evaluation_status: 
#define FIX_RELATIVE_ERROR 0   // fix the relative error
#define FIX_ABSOLUTE_ERROR 1   // fix the absolute error
#define FIX_SUBDIVISIONS 2     // fix the the number of subdivisions
#define FIX_COMBINED 10         // individual setting

// definitions for the variable error_handling:
#define NO_ERROR_REPORT 0
#define ERROR_REPORT 1 // errors are being written to cerr
#define ALL_REPORT 2   // in this case each function call is also reported

/********************************************************************/
/* To allow for an easy adaption of the routines for integration    */
/* of complex functions and/or calculation with increased precision */
/* the type of the result and the type of real numbers are defined  */
/* as RESULT_TYPE and Double                                          */
/********************************************************************/

#include "Cplx.h"	

#define RESULT_TYPE Double
#define QAGCLASS GKInt

class QAGCLASS
{
private:
  int error_handling;
  int evaluation_status;
  int max_subdivisions;
  Double max_absolute_error;
  Double max_relative_error;
  int standard_key;
  RESULT_TYPE *fv1, *fv2;
  int gauss_center,kron_center,
      loop1l,loop2l,loop3l;
  char even_gauss;
  Double *wgk,*wg,*xgk;
  void initialize();
  void create();
  void init_qk15();
  void init_qk21();
  void init_qk31();
  void init_qk41();
  void init_qk51();
  void init_qk61();
  RESULT_TYPE qk(RESULT_TYPE (*f)(Double),Double a,Double b,
                 Double *abserr,Double *resabs,Double *resasc);
  void qsort(int limit,int last,int *maxerr,
             Double *ermax,Double *elist,int *iord,int *nrmax);
  int *iord;
  Double *alist, *blist, *elist;
  RESULT_TYPE *rlist;
public:
  RESULT_TYPE qk15(RESULT_TYPE (*f)(Double),Double a,Double b,
                   Double *abserr,Double *resabs,Double *resasc);
  RESULT_TYPE qk21(RESULT_TYPE (*f)(Double),Double a,Double b,
                   Double *abserr,Double *resabs,Double *resasc);
  RESULT_TYPE qk31(RESULT_TYPE (*f)(Double),Double a,Double b,
                   Double *abserr,Double *resabs,Double *resasc);
  RESULT_TYPE qk41(RESULT_TYPE (*f)(Double),Double a,Double b,
                   Double *abserr,Double *resabs,Double *resasc);
  RESULT_TYPE qk51(RESULT_TYPE (*f)(Double),Double a,Double b,
                   Double *abserr,Double *resabs,Double *resasc);
  RESULT_TYPE qk61(RESULT_TYPE (*f)(Double),Double a,Double b,
                   Double *abserr,Double *resabs,Double *resasc);
  RESULT_TYPE qage(RESULT_TYPE (*f)(Double),Double a,Double b,Double epsabs,
                   Double epsrel,int key,int limit,
                   Double *abserr,int *neval,int *ier,Double *alist,
                   Double *blist,RESULT_TYPE *rlist,Double *elist,
                   int *iord,int *last);
  RESULT_TYPE qagmain(RESULT_TYPE (*f) (Double),Double a,Double b,Double epsabs,
                      Double epsrel,int key,Double *abserr,
                      int *neval,int *ier,int limit,int noisy);
  QAGCLASS(void);
  QAGCLASS(int error_type,Double error);
  QAGCLASS(Double abs_error,Double rel_error,int max_subdiv);
  void set(Double abs_error,Double rel_error,int max_subdiv);
  ~QAGCLASS(void);
  void set_error_handling(int i);
  RESULT_TYPE integrate(RESULT_TYPE (*f) (Double),Double a,Double b);
  Double reported_absolute_error;
  Double reported_relative_error;
  int reported_f_evaluations;
  int reported_error_status;
  void reported_errors(Double *relative,Double *absolute,int *f_evaluations);
};

#endif // ifndef __QagClass__
